﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Text.RegularExpressions;
using Fiddler;
using Ivony.Web.Html;
using Ivony.Web.Html.Parser;
using Ivony.Web.Html.Styles;


[assembly: Fiddler.RequiredVersion( "2.2.8.6" )]

namespace GardenAirPurifier
{

  public class FiddlerExtension : IAutoTamper
  {
    #region IAutoTamper 成员

    public void AutoTamperRequestAfter( Session oSession )
    {
    }

    public void AutoTamperRequestBefore( Session oSession )
    {
    }

    public void AutoTamperResponseAfter( Session oSession )
    {

      oSession.utilDecodeResponse();

      if ( oSession.oRequest.host.EndsWith( "cnblogs.com" ) )
      {
        UriBuilder builder = new UriBuilder();

        builder.Scheme = oSession.oRequest.headers.UriScheme;
        builder.Host = oSession.oRequest.host;
        builder.Path = oSession.oRequest.headers.RequestPath;


        if ( oSession.oResponse.MIMEType.Contains( "text/html" ) )//如果是HTML
        {
          var content = oSession.GetResponseBodyAsString();
          oSession.utilSetResponseBody( CnblogsFilter.Cleanup( builder.Uri, content ) );
        }

        else if ( oSession.oResponse.MIMEType.Contains( "application/json" ) )//如果是JSON
        {
          try
          {
            var serializer = new DataContractJsonSerializer( typeof( CnblogsJsonType ) );

            Encoding encoding = Encoding.UTF8;

            CnblogsJsonType jsonObj;

            using ( var stream = new MemoryStream( encoding.GetBytes( oSession.GetResponseBodyAsString() ) ) )
            {
              jsonObj = (CnblogsJsonType) serializer.ReadObject( stream );
            }

            jsonObj.d = CnblogsFilter.Cleanup( builder.Uri, jsonObj.d );

            using ( var stream = new MemoryStream() )
            {
              serializer.WriteObject( stream, jsonObj );
              oSession.utilSetResponseBody( encoding.GetString( stream.ToArray() ) );
            }

          }
          catch ( Exception e )
          {

            using ( var writer = new StreamWriter( @"C:\Logs\Error.log", true ) )
            {
              writer.WriteLine( builder.Uri );
              writer.WriteLine( new string( '-', 100 ) );
              writer.WriteLine( oSession.GetResponseBodyAsString() );
              writer.WriteLine( new string( '-', 100 ) );
              writer.WriteLine( e );
              writer.WriteLine( new string( '#', 100 ) );
            }
          }
        }
      }
    }

    public void AutoTamperResponseBefore( Session oSession )
    {
    }

    public void OnBeforeReturningError( Session oSession )
    {
    }

    #endregion

    #region IFiddlerExtension 成员

    public void OnBeforeUnload()
    {
    }

    public void OnLoad()
    {
    }

    #endregion
  }

  [DataContract]
  public class CnblogsJsonType
  {
    [DataMember]
    public string d { get; set; }
  }


  public static class CnblogsFilter
  {

    private static readonly Regex userBlogUrlRegex = new Regex( @"(^http://home.cnblogs.com/(?<username>[\w-]+)(/(?<resource>.*))?$)|(^http://www.cnblogs.com/(?<username>[\w-]+)(/(?<resource>.*))?$)|(^http://(?<username>[\w-]+).cnblogs.com(/(?<resource>.*))?$)", RegexOptions.Compiled );

    public static string Cleanup( Uri url, string content )
    {

      var blockedUsersSetting = (string) Microsoft.Win32.Registry.GetValue( @"HKEY_CURRENT_USER\Software\GardenAirPurifier", "blockedUsers", null );
      var recommendUsersSetting = (string) Microsoft.Win32.Registry.GetValue( @"HKEY_CURRENT_USER\Software\GardenAirPurifier", "recommendUsers", null );

      var blockedUsers = new List<string>() { "141853" };//这个是默认的。
      var recommendUsers = new List<string>();

      if ( !string.IsNullOrEmpty( blockedUsersSetting ) )
        blockedUsers.AddRange( blockedUsersSetting.Split( ',' ) );

      if ( !string.IsNullOrEmpty( recommendUsersSetting ) )
        recommendUsers.AddRange( recommendUsersSetting.Split( ',' ) );


      var document = new JumonyParser().Parse( content );

      if ( document == null )
        throw new Exception( "a" );

      var processed = new List<IHtmlElement>();


      string currentUser = null;//当前博客用户

      var currentUrlMatch = userBlogUrlRegex.Match( url.AbsoluteUri );
      if ( currentUrlMatch.Success )
      {
        currentUser = currentUrlMatch.Groups["username"].Value;
      }


      while ( true )
      {
        var attribute = document.Find( "A" )
          .Except( processed )
          .Select( element => element.Attribute( "href" ) )
          .Where( a => a.Value() != null )
          .FirstOrDefault();

        if ( attribute == null )
          break;

        processed.Add( attribute.Element );

        var href = new Uri( url, attribute.Value() );

        /*
        if ( href.Host == url.Host && href.AbsolutePath == url.AbsolutePath )//不处理本页链接
          continue;
        */

        var match = userBlogUrlRegex.Match( href.AbsoluteUri );

        if ( !match.Success )
          continue;

        var username = match.Groups["username"].Value;

        if ( blockedUsers.Contains( username ) )
        {



          if ( match.Groups["resource"].Value == "" )//作者链接
            FindContainer( attribute.Element, false ).Remove();//查找列表项容器删除

          else if ( match.Groups["resource"].Value.StartsWith( "archive/" ) )//博文链接
            FindContainer( attribute.Element, true ).Remove();//查找列表项容器删除

          else
            attribute.Element.Remove();//直接删除这个链接。
        }

        if ( recommendUsers.Contains( username ) )
        {
          if ( username.Equals( currentUser, StringComparison.InvariantCultureIgnoreCase ) )
            continue;

          if ( match.Groups["resource"].Value == "" )//作者链接或者博文链接
          {
            FindContainer( attribute.Element, false ).Style().Set( "border", "dashed 2px #0000AA" );
          }
          attribute.Element.Style().Set( "color", "red" );
          attribute.Element.Style().Set( "font-weight", "bold" );
        }


      }


      return document.OuterHtml();
    }

    private static IHtmlElement FindContainer( IHtmlElement element, bool isDocument )
    {


      var container = element;


      while ( true )
      {
        container = container.Parent as IHtmlElement;
        if ( container == null || container.Parent == null )
          break;


        foreach ( var sibling in container.Siblings().Except( new[] { container } ) )
        {
          var siblingLinks = from link in sibling.Find( "A" )
                             let href = link.Attribute( "href" ).Value()
                             where href != null
                             let match = userBlogUrlRegex.Match( href )
                             where match.Success
                             where !isDocument || match.Groups["resource"].Value.StartsWith( "archive/" )
                             select link;


          foreach ( var link in siblingLinks )
          {

            var path1 = sibling.GetPath( link );
            var path2 = container.GetPath( element );


            if ( path1.Equals( path2, StringComparison.InvariantCultureIgnoreCase ) )
              return container;
          }
        }

      }

      return element;

    }
  }

  public static class Extensions
  {
    public static string GetPath( this IHtmlContainer container, IHtmlElement element )
    {
      StringBuilder builder = new StringBuilder();
      do
      {
        builder.Insert( 0, "/" + element.Name );
        element = element.Parent as IHtmlElement;
        if ( element == null )
          return null;
      }
      while ( element.Equals( container ) );

      return builder.ToString();
    }
  }

}
